@@ -212,6 +212,25 @@ class App(object):
     <i class='mdi mdi-{icon} headerboxicon'></i>
   </a>
 """        
+
+class Link:
+    '''
+    a link
+    '''
+    @staticmethod
+    def create(url,text,tooltip=None):
+        '''
+        create a link for the given url and text
+        
+        Args:
+            url(str): the url
+            text(str): the text
+            tooltip(str): an optional tooltip
+        '''
+        title="" if tooltip is None else f" title='{tooltip}'"
+        link=f"<a href='{url}{title}'>{text}</a>"
+        return link
+
              
 class ComboBox(jp.Input):
     '''
@@ -282,14 +301,98 @@ class Spinner(jp.Div):
     """
     bootstrap5 spinner
     see https://getbootstrap.com/docs/5.0/components/spinners/
-    <div class="spinner-border" role="status">
-      <span class="visually-hidden">Loading...</span>
-    </div>
     """
 
     def __init__(self, **kwargs):
         """
         constructor
         """
-        super(Spinner, self).__init__(classes="spinner-border", role="status", **kwargs)
-        self.span = jp.Span(a=self, classes="visually-hidden", text="Loading...")
\ No newline at end of file
+        classes = "spinner-border"
+        kwargs["classes"] = f"{classes} {kwargs.get('classes', '')}"
+        super(Spinner, self).__init__(role="status", **kwargs)
+        self.span = jp.Span(a=self, classes="visually-hidden", text="Loading...")
+
+class Alert(jp.Div):
+    """
+    Bootstrap 5 Alert
+    see https://getbootstrap.com/docs/5.0/components/alerts/
+    """
+
+    def __init__(self, **kwargs):
+        """
+        constructor
+        """
+        super(Alert, self).__init__(classes="alert alert-warning alert-dismissible fade show", role="alert", **kwargs)
+        conf={
+            "data-bs-dismiss":"alert",
+            "classes":"btn-close",
+            "aria-label":"Close"
+        }
+        self.btn = jp.Button(a=self, **conf)
+        self.btn.on("click", self.delete_alert)
+
+    def delete_alert(self, _msg):
+        """
+        delete the alert message
+        """
+        self.a.remove_component(self)
+        
+class Collapsible(jp.Div):
+    """
+    Collapsible div
+    """
+    button_classes = 'btn btn-primary'
+
+    def __init__(self, label:str, **kwargs):
+        '''
+        constructor
+        '''
+        super().__init__(**kwargs)
+        self.btn = jp.Button(a=self, text=label, classes=self.button_classes)
+        self.body = jp.Div(a=self)
+        self.body.visibility_state = "invisible"
+        self.btn.body=self.body
+        self.body.a = self
+        self.body.classes=''
+        self.on("click", self.toggle_visible)
+
+    @staticmethod
+    def toggle_visible(self, _msg):
+        if self.body.visibility_state == 'visible':
+            self.body.set_class('invisible')
+            self.body.visibility_state = 'invisible'
+        else:
+            self.body.set_class('visible')
+            self.body.visibility_state = 'visible'
+            
+class DebugOutput(jp.Div):
+    """
+    shows debug messages
+    """
+
+    def __init__(self, **kwargs):
+        '''
+        constructor
+        '''
+        super().__init__(**kwargs)
+        self.messages = []
+        self.collapsible = Collapsible("Log", a=self)
+
+    def react(self, _data):
+        '''
+        called on each event
+        '''
+        if self.collapsible:
+            self.collapsible.body.delete()
+        ul = jp.Ul(a=self.collapsible.body, classes="list-none")
+        for msg in reversed(self.messages):
+            jp.Li(a=ul, text=msg)
+
+    def addMessage(self, msg:str):
+        '''
+        add message
+        
+        Args:
+            msg(str): a message to add
+        '''
+        self.messages.append(msg)
